﻿//////////////////////////////////////////////////////////////////////////////
//   Author:        George Potts
//                  Copyright 2015
//   Namespace:     PottsSoftware
//   Class:         SSELibrary
//   Description:   Managed function fallback implementations.
//
//   Initial Date:  2015/08/14
//
//   Notes:         Managed versions of some of the operations.
//
//   Revision History:
//
//////////////////////////////////////////////////////////////////////////////
using System;

namespace PottsSoftware
{
    partial class SSELibrary
    {
        public partial class SafeNativeMethods
        {
            public static SSEDelegate2 Delegate_CS_MEMSET8 = new SSEDelegate2(CS_MEMSET8);
            public static unsafe void CS_MEMSET8(IntPtr dst, ulong bytes, int val)
            {
                ulong blocks = (bytes + sizeof(UInt64) -1) / sizeof(UInt64);

                byte bval = (byte)val;
                UInt64* dulPtr = (UInt64*)dst;
                UInt64 uval = BitConverter.ToUInt64(new byte[] { bval, bval, bval, bval, bval, bval, bval, bval }, 0);

                // This is the only one where using pointer assignments actually improves the speed.
                for (; blocks > 0; blocks--)
                {
                    *dulPtr = uval;
                    dulPtr++;
                }
            } // CS_MEMSET8

            public static SSEDelegate3 Delegate_CS_AND = new SSEDelegate3(CS_AND);
            public static unsafe void CS_AND(IntPtr dst, IntPtr src, ulong bytes)
            {
                ulong blocks = (bytes + sizeof(UInt64) -1) / sizeof(UInt64);

                UInt64* dulPtr = (UInt64*)dst;
                UInt64* sulPtr = (UInt64*)src;

                for (ulong idx = 0; idx < blocks; idx++)
                {
                    dulPtr[idx] = dulPtr[idx] & sulPtr[idx];
                }
            } // CS_AND

            public static SSEDelegate3 Delegate_CS_OR = new SSEDelegate3(CS_OR);
            public static unsafe void CS_OR(IntPtr dst, IntPtr src, ulong bytes)
            {
                ulong blocks = (bytes + sizeof(UInt64) - 1) / sizeof(UInt64);

                UInt64* dulPtr = (UInt64*)dst;
                UInt64* sulPtr = (UInt64*)src;

                for (ulong idx = 0; idx < blocks; idx++)
                {
                    dulPtr[idx] = dulPtr[idx] | sulPtr[idx];
                }
            } // CS_OR

            public static SSEDelegate3 Delegate_CS_XOR = new SSEDelegate3(CS_XOR);
            public static unsafe void CS_XOR(IntPtr dst, IntPtr src, ulong bytes)
            {
                ulong blocks = (bytes + sizeof(UInt64) -1) / sizeof(UInt64);

                UInt64* dulPtr = (UInt64*)dst;
                UInt64* sulPtr = (UInt64*)src;

                for (ulong idx = 0; idx < blocks; idx++)
                {
                    dulPtr[idx] = dulPtr[idx] ^ sulPtr[idx];
                }
            } // CS_XOR

            public static SSEDelegate3 Delegate_CS_ANDNOT = new SSEDelegate3(CS_ANDNOT);
            public static unsafe void CS_ANDNOT(IntPtr dst, IntPtr src, ulong bytes)
            {
                ulong blocks = (bytes + sizeof(UInt64) -1) / sizeof(UInt64);

                UInt64* dulPtr = (UInt64*)dst;
                UInt64* sulPtr = (UInt64*)src;

                for (ulong idx = 0; idx < blocks; idx++)
                {
                    dulPtr[idx] = (~dulPtr[idx]) & sulPtr[idx];
                }
            } // CS_ANDNOT
        } //*** SafeNativeMethods
    } //*** SSELibrary
} //*** PottsSoftware
