﻿//////////////////////////////////////////////////////////////////////////////
///   Namespace:        PottsSoftware
///   Class:            SSELibrary
///   Description:      Managed functions the wrap the calls to the unmanaged
///                     code and handling the conversion and copying of buffers.
///                     the unsafe unmanaged dll.
///   Author:           George Potts
///   Initial Date:     2015/08/13
///   Notes:
///
///   Revision History:
//////////////////////////////////////////////////////////////////////////////

// Expression-Bodied functions, new in C# 6.0, VS2015.
#pragma warning disable CSE0003

namespace PottsSoftware
{
    partial class SSELibrary
    {
        public bool Supports_MMX() { return SafeNativeMethods.Supports_MMX(); } 
        public bool Supports_MMXEXT() { return SafeNativeMethods.Supports_MMXEXT(); } 
        public bool Supports_3DNOW() { return SafeNativeMethods.Supports_3DNOW(); } 
        public bool Supports_3DNOWEXT() { return SafeNativeMethods.Supports_3DNOWEXT(); } 
        public bool Supports_SSE1() { return SafeNativeMethods.Supports_SSE1(); } 
        public bool Supports_SSE2() { return SafeNativeMethods.Supports_SSE2(); } 
        public bool Supports_SSE3() { return SafeNativeMethods.Supports_SSE3(); } 
        public bool Supports_SSSE3() { return SafeNativeMethods.Supports_SSSE3(); } 
        public bool Supports_SSE4A() { return SafeNativeMethods.Supports_SSE4A(); } 
        public bool Supports_SSE41() { return SafeNativeMethods.Supports_SSE41(); } 
        public bool Supports_SSE42() { return SafeNativeMethods.Supports_SSE42(); } 
        public bool Supports_AVX1() { return SafeNativeMethods.Supports_AVX1(); } 
        public bool Supports_AVX2() { return SafeNativeMethods.Supports_AVX2(); }
        public bool Supports_AVX512() { return SafeNativeMethods.Supports_AVX512(); }

        // AND
        public void AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AND); }
        public void SSE_AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_AND); }
        public void AVX_AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX_AND); }
        public void AVX2_AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_AND); }
        public void CPP_AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_AND); }
        public void CS_AND(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CS_AND); }

        // OR
        public void OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_OR); }
        public void SSE_OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_OR); }
        public void AVX_OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX_OR); }
        public void AVX2_OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_OR); }
        public void CPP_OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_OR); }
        public void CS_OR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CS_OR); }

        // XOR
        public void XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_XOR); }
        public void SSE_XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_XOR); }
        public void AVX_XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX_XOR); }
        public void AVX2_XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_XOR); }
        public void CPP_XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_XOR); }
        public void CS_XOR(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CS_XOR); }

        // ANDNOT
        public void ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_ANDNOT); }
        public void SSE_ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_ANDNOT); }
        public void AVX_ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX_ANDNOT); }
        public void AVX2_ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_ANDNOT); }
        public void CPP_ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_ANDNOT); }
        public void CS_ANDNOT(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CS_ANDNOT); }

        // Adds_U8
        public void Adds_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Adds_U8); }
        public void SSE_Adds_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Adds_U8); }
        public void AVX2_Adds_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Adds_U8); }
        public void CPP_Adds_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Adds_U8); }

        // Adds_I8
        public void Adds_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Adds_I8); }
        public void SSE_Adds_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Adds_I8); }
        public void AVX2_Adds_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Adds_I8); }
        public void CPP_Adds_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Adds_I8); }

        // Subs_I8
        public void Subs_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Subs_I8); }
        public void SSE_Subs_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Subs_I8); }
        public void AVX2_Subs_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Subs_I8); }
        public void CPP_Subs_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Subs_I8); }

        // Subs_U8
        public void Subs_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Subs_U8); }
        public void SSE_Subs_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Subs_U8); }
        public void AVX2_Subs_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Subs_U8); }
        public void CPP_Subs_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Subs_U8); }

        // Add_U8
        public void Add_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Add_U8); }
        public void SSE_Add_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Add_U8); }
        public void AVX2_Add_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Add_U8); }
        public void CPP_Add_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Add_U8); }

        // Add_I8
        public void Add_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Add_I8); }
        public void SSE_Add_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Add_I8); }
        public void AVX2_Add_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Add_I8); }
        public void CPP_Add_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Adds_I8); }

        // Sub_I8
        public void Sub_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Sub_I8); }
        public void SSE_Sub_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Sub_I8); }
        public void AVX2_Sub_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Sub_I8); }
        public void CPP_Sub_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Sub_I8); }

        // Sub_U8
        public void Sub_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Sub_U8); }
        public void SSE_Sub_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Sub_U8); }
        public void AVX2_Sub_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Sub_U8); }
        public void CPP_Sub_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Sub_U8); }

        // Avg_U8
        public void Avg_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Avg_U8); }
        public void SSE_Avg_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Avg_U8); }
        public void AVX2_Avg_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Avg_U8); }
        public void CPP_Avg_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Avg_U8); }

        // CmpEQ_I8
        public void CmpEQ_8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpEQ_8); }
        public void SSE_CmpEQ_8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpEQ_8); }
        public void AVX2_CmpEQ_8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpEQ_8); }
        public void CPP_CmpEQ_8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpEQ_8); }

        // CmpGT_I8
        public void CmpGT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpGT_I8); }
        public void SSE_CmpGT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpGT_I8); }
        public void AVX2_CmpGT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpGT_I8); }
        public void CPP_CmpGT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpGT_I8); }

        // CmpGT_U8
        public void CmpGT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpGT_U8); }
        public void SSE_CmpGT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpGT_U8); }
        public void AVX2_CmpGT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpGT_U8); }
        public void CPP_CmpGT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpGT_U8); }

        // CmpLT_I8
        public void CmpLT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpLT_I8); }
        public void SSE_CmpLT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpLT_I8); }
        public void AVX2_CmpLT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpLT_I8); }
        public void CPP_CmpLT_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpLT_I8); }

        // CmpLT_U8
        public void CmpLT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpLT_U8); }
        public void SSE_CmpLT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpLT_U8); }
        public void AVX2_CmpLT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpLT_U8); }
        public void CPP_CmpLT_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpLT_U8); }

        // CmpGTE_U8
        public void CmpGTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpGTE_U8); }
        public void SSE_CmpGTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpGTE_U8); }
        public void AVX2_CmpGTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpGTE_U8); }
        public void CPP_CmpGTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpGTE_U8); }

        // CmpGTE_I8
        public void CmpGTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpGTE_I8); }
        public void SSE_CmpGTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpGTE_I8); }
        public void AVX2_CmpGTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpGTE_I8); }
        public void CPP_CmpGTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpGTE_I8); }

        // CmpLTE_U8
        public void CmpLTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpLTE_U8); }
        public void SSE_CmpLTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpLTE_U8); }
        public void AVX2_CmpLTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpLTE_U8); }
        public void CPP_CmpLTE_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpLTE_U8); }

        // CmpLTE_I8
        public void CmpLTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CmpLTE_I8); }
        public void SSE_CmpLTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_CmpLTE_I8); }
        public void AVX2_CmpLTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_CmpLTE_I8); }
        public void CPP_CmpLTE_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_CmpLTE_I8); }

        // Max_U8
        public void Max_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Max_U8); }
        public void SSE_Max_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Max_U8); }
        public void AVX2_Max_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Max_U8); }
        public void CPP_Max_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Max_U8); }

        // Max_I8
        public void Max_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Max_I8); }
        public void SSE41_Max_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE41_Max_I8); }
        public void AVX2_Max_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Max_I8); }
        public void CPP_Max_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Max_I8); }

        // Min_U8
        public void Min_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Min_U8); }
        public void SSE_Min_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Min_U8); }
        public void AVX2_Min_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Min_U8); }
        public void CPP_Min_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Min_U8); }

        // Min_I8
        public void Min_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Min_I8); }
        public void SSE41_Min_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE41_Min_I8); }
        public void AVX2_Min_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Min_I8); }
        public void CPP_Min_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Min_I8); }

        // Abs_I8
        public void Abs_I8(byte[] dest, ulong bytes) { Operation1(dest, bytes, SafeNativeMethods.Delegate_Abs_I8); }
        public void SSE_Abs_I8(byte[] dest, ulong bytes) { Operation1(dest, bytes, SafeNativeMethods.Delegate_SSE_Abs_I8); }
        public void AVX2_Abs_I8(byte[] dest, ulong bytes) { Operation1(dest, bytes, SafeNativeMethods.Delegate_AVX2_Abs_I8); }
        public void CPP_Abs_I8(byte[] dest, ulong bytes) { Operation1(dest, bytes, SafeNativeMethods.Delegate_CPP_Abs_I8); }

        // Sign_I8
        public void Sign_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Sign_I8); }
        public void SSSE3_Sign_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSSE3_Sign_I8); }
        public void AVX2_Sign_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Sign_I8); }
        public void CPP_Sign_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Sign_I8); }

        // BlendV_I8
        public void BlendV_I8(byte[] dest, byte[] src, byte[] mask, ulong bytes) { Operation3(dest, src, mask, bytes, SafeNativeMethods.Delegate_BlendV_I8); }
        public void SSE_BlendV_I8(byte[] dest, byte[] src, byte[] mask, ulong bytes) { Operation3(dest, src, mask, bytes, SafeNativeMethods.Delegate_SSE_BlendV_I8); }
        public void AVX2_BlendV_I8(byte[] dest, byte[] src, byte[] mask, ulong bytes) { Operation3(dest, src, mask, bytes, SafeNativeMethods.Delegate_AVX2_BlendV_I8); }
        public void CPP_BlendV_I8(byte[] dest, byte[] src, byte[] mask, ulong bytes) { Operation3(dest, src, mask, bytes, SafeNativeMethods.Delegate_CPP_BlendV_I8); }

        // Diff_U8
        public void Diff_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Diff_U8); }
        public void SSE_Diff_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Diff_U8); }
        public void AVX2_Diff_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Diff_U8); }
        public void CPP_Diff_U8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Diff_U8); }

        // Diff_I8
        public void Diff_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_Diff_I8); }
        public void SSE_Diff_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_SSE_Diff_I8); }
        public void AVX2_Diff_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_AVX2_Diff_I8); }
        public void CPP_Diff_I8(byte[] dest, byte[] src, ulong bytes) { Operation2(dest, src, bytes, SafeNativeMethods.Delegate_CPP_Diff_I8); }
    }
}
