﻿//////////////////////////////////////////////////////////////////////////////
///   Namespace:        PottsSoftware
///                     Copyright 2015
///   Class:            SSELibrary
///   Description:      Importer class that links to the dll.
///
///   Author:           George Potts
///   Initial Date:     2015/08/13
///   Notes:
///
///   Revision History:
//////////////////////////////////////////////////////////////////////////////
using System;
using System.Runtime.InteropServices;

namespace PottsSoftware
{
    partial class SSELibrary
    {
        public partial class SafeNativeMethods
        {
            // Simply returns true, any error with the code path will not.
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool IsPresent();

            #region ProcessorSupport
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_MMX();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_MMXEXT();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_3DNOW();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_3DNOWEXT();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE1();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE2();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE3();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSSE3();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE4A();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE41();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_SSE42();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_AVX1();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_AVX2();

            // Not yet implemented, always returns false.
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern bool Supports_AVX512();

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern void DisbleDispatchFeatures( bool avx512, bool avx2, bool avx, bool sse42, bool sse41, bool ssse3, bool sse3);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static extern void EnableDispatchFeatures( bool avx512, bool avx2, bool avx, bool sse42, bool sse41, bool ssse3, bool sse3);
            #endregion // ProcessorSupport

            #region MEMSET
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void MEMSET8(IntPtr dst, ulong bytes, int val);
            public static SSEDelegate2 Delegate_MEMSET8 = new SSEDelegate2(MEMSET8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_MEMSET8(IntPtr dst, ulong bytes, int val);
            public static SSEDelegate2 Delegate_SSE_MEMSET8 = new SSEDelegate2(SSE_MEMSET8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX_MEMSET8(IntPtr dst, ulong bytes, int val);
            public static SSEDelegate2 Delegate_AVX_MEMSET8 = new SSEDelegate2(AVX_MEMSET8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_MEMSET8(IntPtr dst, ulong bytes, int val);
            public static SSEDelegate2 Delegate_CPP_MEMSET8 = new SSEDelegate2(CPP_MEMSET8);
            #endregion MEMSET

            #region LogicalOperators_AND_OR_XOR_ANDNOT
            // AND
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AND(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AND = new SSEDelegate3(AND);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX_AND(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX_AND = new SSEDelegate3(AVX_AND);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_AND(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_AND = new SSEDelegate3(AVX2_AND);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_AND(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_AND = new SSEDelegate3(SSE_AND);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_AND(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_AND = new SSEDelegate3(CPP_AND);

            // OR
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void OR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_OR = new SSEDelegate3(OR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_OR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_OR = new SSEDelegate3(SSE_OR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX_OR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX_OR = new SSEDelegate3(AVX_OR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_OR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_OR = new SSEDelegate3(AVX2_OR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_OR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_OR = new SSEDelegate3(CPP_OR);

            // XOR
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void XOR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_XOR = new SSEDelegate3(XOR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_XOR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_XOR = new SSEDelegate3(SSE_XOR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX_XOR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX_XOR = new SSEDelegate3(AVX_XOR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_XOR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_XOR = new SSEDelegate3(AVX2_XOR);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_XOR(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_XOR = new SSEDelegate3(CPP_XOR);

            // ANDNOT
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void ANDNOT(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_ANDNOT = new SSEDelegate3(ANDNOT);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_ANDNOT(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_ANDNOT = new SSEDelegate3(SSE_ANDNOT);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX_ANDNOT(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX_ANDNOT = new SSEDelegate3(AVX_ANDNOT);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_ANDNOT(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_ANDNOT = new SSEDelegate3(AVX2_ANDNOT);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_ANDNOT(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_ANDNOT = new SSEDelegate3(CPP_ANDNOT);
            #endregion LogicalOperators_AND_OR_XOR_ANDNOT

            #region Adds_Subs
            // Adds_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Adds_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Adds_U8 = new SSEDelegate3(Adds_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Adds_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Adds_U8 = new SSEDelegate3(SSE_Adds_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Adds_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Adds_U8 = new SSEDelegate3(AVX2_Adds_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Adds_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Adds_U8 = new SSEDelegate3(CPP_Adds_U8);

            //Adds_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Adds_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Adds_I8 = new SSEDelegate3(Adds_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Adds_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Adds_I8 = new SSEDelegate3(SSE_Adds_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Adds_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Adds_I8 = new SSEDelegate3(AVX2_Adds_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Adds_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Adds_I8 = new SSEDelegate3(CPP_Adds_I8);

            // Subs_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Subs_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Subs_U8 = new SSEDelegate3(Subs_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Subs_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Subs_U8 = new SSEDelegate3(SSE_Subs_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Subs_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Subs_U8 = new SSEDelegate3(AVX2_Subs_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Subs_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Subs_U8 = new SSEDelegate3(CPP_Subs_U8);

            //Subs_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Subs_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Subs_I8 = new SSEDelegate3(Subs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Subs_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Subs_I8 = new SSEDelegate3(SSE_Subs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Subs_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Subs_I8 = new SSEDelegate3(AVX2_Subs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Subs_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Subs_I8 = new SSEDelegate3(CPP_Subs_I8);
            #endregion // Add_Sub

            #region Add_Sub
            // Add_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Add_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Add_U8 = new SSEDelegate3(Add_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Add_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Add_U8 = new SSEDelegate3(SSE_Add_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Add_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Add_U8 = new SSEDelegate3(AVX2_Add_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Add_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Add_U8 = new SSEDelegate3(CPP_Add_U8);

            //Add_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Add_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Add_I8 = new SSEDelegate3(Add_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Add_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Add_I8 = new SSEDelegate3(SSE_Add_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Add_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Add_I8 = new SSEDelegate3(AVX2_Add_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Add_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Add_I8 = new SSEDelegate3(CPP_Add_I8);

            // Sub_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Sub_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Sub_U8 = new SSEDelegate3(Sub_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Sub_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Sub_U8 = new SSEDelegate3(SSE_Sub_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Sub_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Sub_U8 = new SSEDelegate3(AVX2_Sub_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Sub_U8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Sub_U8 = new SSEDelegate3(CPP_Sub_U8);

            //Sub_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Sub_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Sub_I8 = new SSEDelegate3(Sub_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Sub_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Sub_I8 = new SSEDelegate3(SSE_Sub_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Sub_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Sub_I8 = new SSEDelegate3(AVX2_Sub_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Sub_I8(IntPtr dest, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Sub_I8 = new SSEDelegate3(CPP_Sub_I8);
            #endregion Add_Sub

            #region Avg
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Avg_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Avg_U8 = new SSEDelegate3(Avg_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Avg_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Avg_U8 = new SSEDelegate3(SSE_Avg_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Avg_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Avg_U8 = new SSEDelegate3(AVX2_Avg_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Avg_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Avg_U8 = new SSEDelegate3(CPP_Avg_U8);
            #endregion Avg

            #region CmpEQ
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpEQ_8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpEQ_8 = new SSEDelegate3(CmpEQ_8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpEQ_8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpEQ_8 = new SSEDelegate3(SSE_CmpEQ_8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpEQ_8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpEQ_8 = new SSEDelegate3(AVX2_CmpEQ_8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpEQ_8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpEQ_8 = new SSEDelegate3(CPP_CmpEQ_8);
            #endregion CmpEQ

            #region CmpGT
            // I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpGT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpGT_I8 = new SSEDelegate3(CmpGT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpGT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpGT_I8 = new SSEDelegate3(SSE_CmpGT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpGT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpGT_I8 = new SSEDelegate3(AVX2_CmpGT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpGT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpGT_I8 = new SSEDelegate3(CPP_CmpGT_I8);

            // U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpGT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpGT_U8 = new SSEDelegate3(CmpGT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpGT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpGT_U8 = new SSEDelegate3(SSE_CmpGT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpGT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpGT_U8 = new SSEDelegate3(AVX2_CmpGT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpGT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpGT_U8 = new SSEDelegate3(CPP_CmpGT_U8);
            #endregion CmpGT

            #region CmpLT
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpLT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpLT_I8 = new SSEDelegate3(CmpLT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpLT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpLT_I8 = new SSEDelegate3(SSE_CmpLT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpLT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpLT_I8 = new SSEDelegate3(AVX2_CmpLT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpLT_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpLT_I8 = new SSEDelegate3(CPP_CmpLT_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpLT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpLT_U8 = new SSEDelegate3(CmpLT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpLT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpLT_U8 = new SSEDelegate3(SSE_CmpLT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpLT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpLT_U8 = new SSEDelegate3(AVX2_CmpLT_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpLT_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpLT_U8 = new SSEDelegate3(CPP_CmpLT_U8);
            #endregion CmpLT

            #region CmpGTE
            // U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpGTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpGTE_U8 = new SSEDelegate3(CmpGTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpGTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpGTE_U8 = new SSEDelegate3(SSE_CmpGTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpGTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpGTE_U8 = new SSEDelegate3(AVX2_CmpGTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpGTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpGTE_U8 = new SSEDelegate3(CPP_CmpGTE_U8);

            //I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpGTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpGTE_I8 = new SSEDelegate3(CmpGTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpGTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpGTE_I8 = new SSEDelegate3(SSE_CmpGTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpGTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpGTE_I8 = new SSEDelegate3(AVX2_CmpGTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpGTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpGTE_I8 = new SSEDelegate3(CPP_CmpGTE_I8);
            #endregion CmpGTE

            #region CmpLTE
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpLTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpLTE_U8 = new SSEDelegate3(CmpLTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpLTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpLTE_U8 = new SSEDelegate3(SSE_CmpLTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpLTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpLTE_U8 = new SSEDelegate3(AVX2_CmpLTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpLTE_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpLTE_U8 = new SSEDelegate3(CPP_CmpLTE_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CmpLTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CmpLTE_I8 = new SSEDelegate3(CmpLTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_CmpLTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_CmpLTE_I8 = new SSEDelegate3(SSE_CmpLTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_CmpLTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_CmpLTE_I8 = new SSEDelegate3(AVX2_CmpLTE_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_CmpLTE_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_CmpLTE_I8 = new SSEDelegate3(CPP_CmpLTE_I8);
            #endregion CmpLTE

            #region Min_Max
            // Max_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Max_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Max_U8 = new SSEDelegate3(Max_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Max_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Max_U8 = new SSEDelegate3(SSE_Max_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Max_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Max_U8 = new SSEDelegate3(AVX2_Max_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Max_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Max_U8 = new SSEDelegate3(CPP_Max_U8);

            // Max_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Max_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Max_I8 = new SSEDelegate3(Max_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE41_Max_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE41_Max_I8 = new SSEDelegate3(SSE41_Max_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Max_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Max_I8 = new SSEDelegate3(AVX2_Max_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Max_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Max_I8 = new SSEDelegate3(CPP_Max_I8);

            // Min_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Min_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Min_U8 = new SSEDelegate3(Min_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Min_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Min_U8 = new SSEDelegate3(SSE_Min_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Min_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Min_U8 = new SSEDelegate3(AVX2_Min_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Min_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Min_U8 = new SSEDelegate3(CPP_Min_U8);

            // Min_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Min_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Min_I8 = new SSEDelegate3(Min_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE41_Min_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE41_Min_I8 = new SSEDelegate3(SSE41_Min_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Min_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Min_I8 = new SSEDelegate3(AVX2_Min_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Min_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Min_I8 = new SSEDelegate3(CPP_Min_I8);
            #endregion Min_Max

            #region Diff
            // Diff_U8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Diff_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Diff_U8 = new SSEDelegate3(Diff_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Diff_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Diff_U8 = new SSEDelegate3(SSE_Diff_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Diff_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Diff_U8 = new SSEDelegate3(CPP_Diff_U8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Diff_U8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Diff_U8 = new SSEDelegate3(AVX2_Diff_U8);

            // Diff_I8
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Diff_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Diff_I8 = new SSEDelegate3(Diff_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Diff_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSE_Diff_I8 = new SSEDelegate3(SSE_Diff_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Diff_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Diff_I8 = new SSEDelegate3(CPP_Diff_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Diff_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Diff_I8 = new SSEDelegate3(AVX2_Diff_I8);
            #endregion Diff

            #region Abs_Sign_Blend
            // Abs
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Abs_I8(IntPtr src, ulong bytes);
            public static SSEDelegate1 Delegate_Abs_I8 = new SSEDelegate1(Abs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_Abs_I8(IntPtr src, ulong bytes);
            public static SSEDelegate1 Delegate_SSE_Abs_I8 = new SSEDelegate1(SSE_Abs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Abs_I8(IntPtr src, ulong bytes);
            public static SSEDelegate1 Delegate_AVX2_Abs_I8 = new SSEDelegate1(AVX2_Abs_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Abs_I8(IntPtr src, ulong bytes);
            public static SSEDelegate1 Delegate_CPP_Abs_I8 = new SSEDelegate1(CPP_Abs_I8);

            // Sign
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void Sign_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_Sign_I8 = new SSEDelegate3(Sign_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSSE3_Sign_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_SSSE3_Sign_I8 = new SSEDelegate3(SSSE3_Sign_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_Sign_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_CPP_Sign_I8 = new SSEDelegate3(CPP_Sign_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_Sign_I8(IntPtr dst, IntPtr src, ulong bytes);
            public static SSEDelegate3 Delegate_AVX2_Sign_I8 = new SSEDelegate3(AVX2_Sign_I8);

            // BlendV
            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void BlendV_I8(IntPtr dst, IntPtr src, IntPtr mask, ulong bytes);
            public static SSEDelegate4 Delegate_BlendV_I8 = new SSEDelegate4(BlendV_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void SSE_BlendV_I8(IntPtr dst, IntPtr src, IntPtr mask, ulong bytes);
            public static SSEDelegate4 Delegate_SSE_BlendV_I8 = new SSEDelegate4(SSE_BlendV_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void CPP_BlendV_I8(IntPtr dst, IntPtr src, IntPtr mask, ulong bytes);
            public static SSEDelegate4 Delegate_CPP_BlendV_I8 = new SSEDelegate4(CPP_BlendV_I8);

            [DllImport("SSELibrary.dll", CallingConvention = CallingConvention.Cdecl)]
            public static unsafe extern void AVX2_BlendV_I8(IntPtr dst, IntPtr src, IntPtr mask, ulong bytes);
            public static SSEDelegate4 Delegate_AVX2_BlendV_I8 = new SSEDelegate4(AVX2_BlendV_I8);
            #endregion 

        } //*** SafeNativeMethods
    } //*** SSELibrary
} //*** namespace
