//////////////////////////////////////////////////////////////////////////////
//   Author:       George Potts
//   Copyright:    PottsSoftware
//   Initial Date: 2015/08/13
// 
//   Description:  Exported functions
//
//   Notes:
//
//   Revision History:
//
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

extern "C" {
    SSELIBRARY_API bool IsPresent();

    // CPUID information.
    SSELIBRARY_API bool Supports_MMX();
    SSELIBRARY_API bool Supports_MMXEXT();
    SSELIBRARY_API bool Supports_3DNOW();
    SSELIBRARY_API bool Supports_3DNOWEXT();
    SSELIBRARY_API bool Supports_SSE1();
    SSELIBRARY_API bool Supports_SSE2();
    SSELIBRARY_API bool Supports_SSE3();
    SSELIBRARY_API bool Supports_SSSE3();
    SSELIBRARY_API bool Supports_SSE4A();
    SSELIBRARY_API bool Supports_SSE41();
    SSELIBRARY_API bool Supports_SSE42();
    SSELIBRARY_API bool Supports_AVX1();
    SSELIBRARY_API bool Supports_AVX2();
    SSELIBRARY_API bool Supports_AVX512();

    SSELIBRARY_API void DisbleDispatchFeatures( bool avx512, bool avx2, bool avx, bool sse42, bool sse41, bool ssse3, bool sse3);
    SSELIBRARY_API void EnableDispatchFeatures( bool avx512, bool avx2, bool avx, bool sse42, bool sse41, bool ssse3, bool sse3);

    // MEMSET
    SSELIBRARY_API void MEMSET8(char* dst, size_t bytes, int val);
    SSELIBRARY_API void SSE_MEMSET8(char* dst, size_t bytes, int val);
    SSELIBRARY_API void AVX_MEMSET8(char* dst, size_t bytes, int val);
    SSELIBRARY_API void CPP_MEMSET8(char* dst, size_t bytes, int val);

    // https://software.intel.com/sites/landingpage/IntrinsicsGuide/
    // Logical bit operations
    SSELIBRARY_API void AND(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_AND(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX_AND(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_AND(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_AND(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void OR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_OR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX_OR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_OR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_OR(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void XOR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_XOR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX_XOR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_XOR(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_XOR(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void ANDNOT(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_ANDNOT(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX_ANDNOT(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_ANDNOT(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_ANDNOT(char* dst, char* src, size_t bytes);

	// 8-bit Saturation operations
    SSELIBRARY_API void Adds_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Adds_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Adds_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Adds_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Adds_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Adds_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Adds_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Adds_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Subs_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Subs_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Subs_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Subs_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Subs_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Subs_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Subs_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Subs_U8(char* dst, char* src, size_t bytes);

	// 8-bit arithmatic
    SSELIBRARY_API void Add_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Add_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Add_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Add_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Add_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Add_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Add_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Add_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Sub_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Sub_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Sub_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Sub_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Sub_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Sub_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Sub_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Sub_U8(char* dst, char* src, size_t bytes);

    // Avg
    SSELIBRARY_API void Avg_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Avg_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Avg_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Avg_U8(char* dst, char* src, size_t bytes);

    // CmpEQ
    SSELIBRARY_API void CmpEQ_8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpEQ_8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpEQ_8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpEQ_8(char* dst, char* src, size_t bytes);

    // CmpGT
    SSELIBRARY_API void CmpGT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpGT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpGT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpGT_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void CmpGT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpGT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpGT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpGT_U8(char* dst, char* src, size_t bytes);

    // CmpLT
    SSELIBRARY_API void CmpLT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpLT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpLT_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpLT_I8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void CmpLT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpLT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpLT_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpLT_U8(char* dst, char* src, size_t bytes);

    // CmpGTE
    SSELIBRARY_API void CmpGTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpGTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpGTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpGTE_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void CmpGTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpGTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpGTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpGTE_I8(char* dst, char* src, size_t bytes);

    // CmpLTE
    SSELIBRARY_API void CmpLTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpLTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpLTE_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpLTE_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void CmpLTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_CmpLTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_CmpLTE_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_CmpLTE_I8(char* dst, char* src, size_t bytes);

    // Max
    SSELIBRARY_API void Max_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Max_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Max_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Max_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Max_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE41_Max_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Max_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Max_I8(char* dst, char* src, size_t bytes);

    // Min
    SSELIBRARY_API void Min_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Min_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Min_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Min_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Min_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE41_Min_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Min_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Min_I8(char* dst, char* src, size_t bytes);

    // Abs
    SSELIBRARY_API void Abs_I8(char* src, size_t bytes);
    SSELIBRARY_API void SSE_Abs_I8(char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Abs_I8(char* src, size_t bytes);
    SSELIBRARY_API void CPP_Abs_I8(char* src, size_t bytes);

    // Sign
    SSELIBRARY_API void Sign_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSSE3_Sign_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Sign_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Sign_I8(char* dst, char* src, size_t bytes);

    // BlendV
    SSELIBRARY_API void BlendV_I8(char* dst, char* src, char* mask, size_t bytes);
    SSELIBRARY_API void SSE_BlendV_I8(char* dst, char* src, char* mask, size_t bytes);
    SSELIBRARY_API void AVX2_BlendV_I8(char* dst, char* src, char* mask, size_t bytes);
    SSELIBRARY_API void CPP_BlendV_I8(char* dst, char* src, char* mask, size_t bytes);

    // Diff
    SSELIBRARY_API void Diff_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Diff_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Diff_U8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Diff_U8(char* dst, char* src, size_t bytes);

    SSELIBRARY_API void Diff_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void SSE_Diff_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void AVX2_Diff_I8(char* dst, char* src, size_t bytes);
    SSELIBRARY_API void CPP_Diff_I8(char* dst, char* src, size_t bytes);
}